@@ -8,10 +8,12 @@ if (!is.R()) {
   }
 }
 
-"CRS" <- function(projargs) {
-    if (missing(projargs)) projargs <- as.character(NA)
-    if (nchar(projargs) == 0) projargs <- as.character(NA)
+"CRS" <- function(projargs=NA_character_, doCheckCRSArgs=TRUE) {
+# cautious change BDR 150424
+    if (!is.na(projargs) && !nzchar(projargs)) projargs <- NA_character_
 # condition added 140301
+    stopifnot(is.logical(doCheckCRSArgs))
+    stopifnot(length(doCheckCRSArgs) == 1L)
     stopifnot(is.character(projargs))
     if (!is.na(projargs)) {
         if (length(grep("^[ ]*\\+", projargs)) == 0)
@@ -39,11 +41,13 @@ if (!is.R()) {
 #    if (length(grep("rgdal", search()) > 0) &&
 #      (sessionInfo()$otherPkgs$rgdal$Version > "0.4-2")) {
 # sessionInfo()/read.dcf() problem in loop 080307
-    if (!is.na(uprojargs) && "rgdal" %in% .packages()) {
-	res <- checkCRSArgs(uprojargs)
-	if (!res[[1]]) 
-	    stop(res[[2]])
+    if (doCheckCRSArgs) {
+      if (!is.na(uprojargs) && requireNamespace("rgdal", quietly = TRUE)) {
+        res <- rgdal::checkCRSArgs(uprojargs)
+        if (!res[[1]]) 
+            stop(res[[2]])
         uprojargs <- res[[2]]
+      }
     }
     res <- new("CRS", projargs=uprojargs)
     res
