@@ -1,21 +1,29 @@
 "SpatialPointsDataFrame" = function(coords, data, coords.nrs = numeric(0), 
-		proj4string = CRS(as.character(NA)), match.ID = TRUE,
-                bbox=NULL) {
-    if (is.character(match.ID)) {
+		proj4string = CRS(as.character(NA)), match.ID, bbox = NULL) {
+
+	if (!is(coords, "SpatialPoints"))
+		coords = coordinates(coords) 
+		# make sure data.frame becomes double matrix; NA checks
+	mtch = NULL
+	cc.ID = row.names(coords) # row.names works for both matrix AND SpatialPoints
+	if (missing(match.ID)) { # sort it out:
+		if (is.null(cc.ID))
+			match.ID = FALSE # nothing to match to!
+		else {
+			mtch = match(cc.ID, row.names(data))
+			match.ID = !any(is.na(mtch)) # && length(unique(mtch)) == nrow(data)
+			if (match.ID && any(mtch != 1:nrow(data)))
+				warning("forming a SpatialPointsDataFrame based on maching IDs, not on record order. Use match.ID = FALSE to match on record order")
+		}
+	} else if (is.character(match.ID)) {
         row.names(data) = data[, match.ID[1]]
         match.ID = TRUE
-    }
-	if (!is(coords, "SpatialPoints"))
-		coords = coordinates(coords)
-	cc.ID = dimnames(coords)[[1]]
-	if (is.null(cc.ID))
-		match.ID = FALSE # nothing to match to!
-	else if (match.ID && length(unique(cc.ID)) != nrow(data))
-		stop("nr of unique coords ID's (rownames) not equal to nr of data records")
+    } 
+
 	if (match.ID) {
 		if (!is.null(cc.ID) && is(data, "data.frame")) { # match ID:
-			data.ID = row.names(data)
-			mtch = match(cc.ID, data.ID)
+			if (is.null(mtch))
+				mtch = match(cc.ID, row.names(data))
 			if (any(is.na(mtch)))
 				stop("row.names of data and coords do not match")
 			if (length(unique(mtch)) != nrow(data))
@@ -24,8 +32,7 @@
 		}
 	}
 	if (!is(coords, "SpatialPoints"))
-		coords = SpatialPoints(coords, proj4string = proj4string, 
-			bbox=bbox)
+		coords = SpatialPoints(coords, proj4string = proj4string, bbox = bbox)
 	# EJP, Tue Aug 13 19:54:04 CEST 2013
 	if (is.character(attr(data, "row.names"))) # i.e., data has "real" row names
 		dimnames(coords@coords)[[1]] = row.names(data)
@@ -43,7 +50,7 @@ setReplaceMethod("coordinates", signature(object = "data.frame", value = "ANY"),
   function(object, value) {
 	coord.numbers = NULL
 	if (inherits(value, "formula")) {
-		cc = model.frame(value, object) # retrieve coords
+		cc = model.frame(value, object, na.action = na.fail) # retrieve coords
 		if (dim(cc)[2] == 2) {
 			nm = as.character(as.list(value)[[2]])[2:3]
 			coord.numbers = match(nm, names(object))
@@ -108,9 +115,9 @@ as.data.frame.SpatialPointsDataFrame = function(x, ...)  {
 			ret[[idx.new[i]]] = x@data[,i]
 		names(ret)[idx.new] = names(x@data)
 		ret = ret[unlist(lapply(ret, function(x) !is.null(x)))]
-		data.frame(ret)
+		data.frame(ret, ...)
 	} else
-		data.frame(x@data, x@coords)
+		data.frame(x@data, x@coords, ...)
 }
 
 setAs("SpatialPointsDataFrame", "data.frame", function(from)
@@ -127,13 +134,24 @@ points.SpatialPointsDataFrame = function(x, y = NULL, ...)
 	points(as(x, "SpatialPoints"), ...)
 
 text.SpatialPointsDataFrame = function(x, ...) {
-    lst = list(x = coordinates(x), ...)
+	lst = list(...)
+	if (!is.null(x$srt)) {
+		if (length(unique(x$srt)) == 1)
+			lst$srt = x$srt[1]
+		else { # print each label individually:
+			lapply(seq_len(length(x)), function(i) text(x[i,], ...))
+			return(invisible())
+		}
+	}
+    lst$x = coordinates(x)
     if (!is.null(x$pos) && is.null(lst$pos))
         lst$pos = x$pos
     if (!is.null(x$offset) && is.null(lst$offset))
         lst$offset = x$offset
     if (!is.null(x$labels) && is.null(lst$labels))
-        lst$labels = parse(text = x$lab)
+        lst$labels = parse(text = x$labels)
+    if (!is.null(x$adjx) && !is.null(x$adjy))
+        lst$adj = c(x$adjx, x$adjy)
     do.call(text, lst)
 }
 
@@ -177,7 +195,8 @@ setMethod("[", "SpatialPointsDataFrame", function(x, i, j, ..., drop = TRUE) {
 	if (!isTRUE(j)) # i.e., we do some sort of column selection
 		x@coords.nrs = numeric(0) # will move coordinate colums last
 	x@coords = x@coords[i, , drop = FALSE]
-	x@bbox = .bboxCoords(x@coords)
+	if (nrow(x@coords))
+		x@bbox = .bboxCoords(x@coords)
 	x@data = x@data[i, j, ..., drop = FALSE]
 	x
 })
